unit Server.Core.Listener;

interface

uses
  System.SysUtils, System.Classes,

  WiRL.Core.Engine,
  WiRL.http.FileSystemEngine,
  WiRL.http.Server,
  WiRL.http.Server.Indy;

type
  TListener = class
  private
    FServer: TWiRLServer;
    FPort: Integer;
    FActive: Boolean;
    procedure SetActive(const Value: Boolean);
  public
    procedure Start;
    procedure Stop;
    property Port: Integer read FPort write FPort;
    property Active: Boolean read FActive write SetActive;
    constructor Create;
  end;

implementation

{ TListener }

constructor TListener.Create;
begin
  FPort := 8080;
end;

procedure TListener.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
  begin
    if Value then
      Stop
    else
      Start;
  end;
end;

procedure TListener.Start;
begin
  // Create http server
  FServer := TWiRLServer.Create(nil);

  // Engine configuration
  FServer
    .SetPort(FPort)
    .SetThreadPoolSize(5);

  FServer
    .AddEngine<TWiRLEngine>('/rest')
      .SetEngineName('WiRL Template')

      // Application configuration
      .AddApplication('/default')
        .SetAppName('Default')
        .SetResources('*');

  FServer
    .AddEngine<TWiRLFileSystemEngine>('/ext76')
    .SetRootFolder('{AppPath}\..\ext76');

  FServer
    .AddEngine<TWiRLFileSystemEngine>('/build')
    .SetRootFolder('{AppPath}\..\build\');

  FServer
    .AddEngine<TWiRLFileSystemEngine>('/packages')
    .SetRootFolder('{AppPath}\..\packages');

  FServer
    .AddEngine<TWiRLFileSystemEngine>('/themes')
    .SetRootFolder('{AppPath}\client\themes');

  FServer
    .AddEngine<TWiRLFileSystemEngine>('/')
    .SetRootFolder('{AppPath}\www');


  if not FServer.Active then
    FServer.Active := True;

  FActive := True;
end;

procedure TListener.Stop;
begin
  FServer.Active := False;
  FreeAndNil(FServer);

  FActive := False;
end;

end.
