unit Server.Core.Service;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes,
  Vcl.Controls, Vcl.SvcMgr, System.Win.Registry, Dialogs,

  Server.Core.Listener;

type
  TmodService = class(TService)
    procedure ServiceCreate(Sender: TObject);
    procedure ServiceExecute(Sender: TService);
    procedure ServiceAfterInstall(Sender: TService);
  private
    FListener: TListener;
  public
    function GetServiceController: TServiceController; override;
  end;

var
  modService: TmodService;

implementation

const
  DefaultServiceName = 'DelphiDayDemo';
  DefaultServiceDisplayName = 'DelphiDay - Demo';
  DefaultServiceDescription = 'Servizio DEMO per il DelphiDay';

{$R *.dfm}

procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  modService.Controller(CtrlCode);
end;

function TmodService.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;

procedure TmodService.ServiceAfterInstall(Sender: TService);
var
  LServiceName: string;
  LRegistry :TRegistry;
  LImageParams :string;
  LPort: string;
  LSuffix: string;
begin
  LServiceName := DefaultServiceName;
  LImageParams := '';
  if FindCmdLineSwitch('port', LPort) then
    LImageParams := LImageParams + ' /port ' + LPort;
  if FindCmdLineSwitch('name', LSuffix) then
    LImageParams := LImageParams + ' /name ' + LSuffix;

  LRegistry := TRegistry.Create;
  try
    LRegistry.RootKey := HKEY_LOCAL_MACHINE;
    if LRegistry.OpenKey('SYSTEM\CurrentControlSet\Services\' + Self.Name, True) then
    begin
      LRegistry.WriteExpandString('ImagePath', ParamStr(0) + LImageParams);
      LRegistry.WriteString('Description', DefaultServiceDescription);
      LRegistry.CloseKey;
    end;
  finally
    LRegistry.Free;
  end;
end;

procedure TmodService.ServiceCreate(Sender: TObject);
var
  LSuffix: string;
begin
  if FindCmdLineSwitch('name', LSuffix) then
  begin
    Name := DefaultServiceName + '_' + LSuffix;
    DisplayName := DefaultServiceDisplayName + ' (' + LSuffix + ')';
  end
  else
  begin
    Name := DefaultServiceName;
    DisplayName := DefaultServiceDisplayName;
  end;
end;

procedure TmodService.ServiceExecute(Sender: TService);
var
  LPort: string;
begin
  FListener := TListener.Create;
  try

    if FindCmdLineSwitch('port', LPort) then
      FListener.Port := StrToIntDef(LPort, 8080);

    FListener.Start;
    while not Terminated do
    begin
      if Assigned(ServiceThread) then
        ServiceThread.ProcessRequests(False);
      Sleep(1000);
    end;
    FListener.Stop;
  finally
    FListener.Free;
  end;
end;

end.
