unit Server.Database.Data;

interface

uses
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Error, FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def,
  FireDAC.Stan.Pool, FireDAC.Stan.Async, FireDAC.Phys, FireDAC.Phys.SQLite,
  FireDAC.Phys.SQLiteDef, FireDAC.Stan.ExprFuncs,
  FireDAC.Phys.SQLiteWrapper.Stat, FireDAC.ConsoleUI.Wait, FireDAC.Stan.Param,
  FireDAC.DatS, FireDAC.DApt.Intf, FireDAC.DApt, Data.DB, FireDAC.Comp.DataSet,
  FireDAC.Comp.Client,

  WiRL.Core.Auth.Context, WiRL.Core.Auth.Resource, FireDAC.VCLUI.Wait;

type
  TDMData = class(TDataModule)
    FDConnection: TFDConnection;
    qryEmployee: TFDQuery;
    qryEmployeeEMP_NO: TSmallintField;
    qryEmployeeFIRST_NAME: TStringField;
    qryEmployeeLAST_NAME: TStringField;
    qryEmployeePHONE_EXT: TStringField;
    qryEmployeeHIRE_DATE: TSQLTimeStampField;
    qryEmployeeDEPT_NO: TStringField;
    qryEmployeeJOB_CODE: TStringField;
    qryEmployeeJOB_GRADE: TSmallintField;
    qryEmployeeJOB_COUNTRY: TStringField;
    qryEmployeeSALARY: TBCDField;
    qryEmpNoGen: TFDQuery;
    qryUsers: TFDQuery;
    qryUpdateJobs: TFDQuery;
    qryGetLastId: TFDQuery;
    qryInsertJob: TFDQuery;
    qryJobs: TFDQuery;
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }
  public
    function GetHashedPassword(const AValue: string): string;
    function CheckPassword(const APassword, AHash: string): Boolean;
    function Authenticate(const AUserName, APassword: string; ASubject: TWiRLSubject): TWiRLAuthResult;
    function GetEmployee(const Filter: string): TDataSet;

    function NewJob(const AOperation: string) :Integer;
    procedure RunJob(JobId: Integer);
    function GetJobs: TDataSet;
  end;

var
  DMData: TDMData;

implementation

uses
  System.NetEncoding, Bcrypt;

const
    INIT_ROUNDS = 10;
    INIT_CODE = 'Bv1#v0ws"k5g';

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

function TDMData.GetEmployee(const Filter: string): TDataSet;
begin
  if filter <> '' then
  begin
    qryEmployee.MacroByName('filter').Value :=
      'WHERE FIRST_NAME LIKE ' + QuotedStr('%' + filter + '%') + ' OR ' +
      'LAST_NAME LIKE ' + QuotedStr('%' + filter + '%');
  end;
  Result := qryEmployee;
end;

function TDMData.GetHashedPassword(const AValue: string): string;
begin
  Result := TBCrypt.HashPassword(INIT_CODE + AValue, INIT_ROUNDS);
  Result := TNetEncoding.Base64URL.Encode(Result);
end;

function TDMData.GetJobs: TDataSet;
begin
  qryJobs.Open;
  Result := qryJobs;
end;

function TDMData.NewJob(const AOperation: string): Integer;
begin
  qryInsertJob.ParamByName('PARAMS').AsString := AOperation;
  qryInsertJob.ExecSQL;
  qryGetLastId.Open;
  if qryGetLastId.IsEmpty then
    raise Exception.Create('qryInsertJob: Argh!');
  Result := qryGetLastId.Fields[0].AsInteger;
end;

procedure TDMData.RunJob(JobId: Integer);
begin
  Sleep(10000);
  // Update Status
  qryUpdateJobs.ParamByName('ID').AsInteger := JobId;
  qryUpdateJobs.Execute()
end;

function TDMData.CheckPassword(const APassword, AHash: string): Boolean;
var
  LHash: string;
  LRehash: Boolean;
begin
  LHash := TNetEncoding.Base64URL.Decode(AHash);
  Result := TBCrypt.CheckPassword(INIT_CODE + APassword, LHash, LRehash);
end;

function TDMData.Authenticate(const AUserName, APassword: string;
  ASubject: TWiRLSubject): TWiRLAuthResult;
begin
  Result.Success := False;
  Result.Roles := [];

  qryUsers.ParamByName('USERNAME').AsString := UpperCase(AUserName);

  qryUsers.Open;

  if qryUsers.IsEmpty then
    Exit;

  Result.Success := CheckPassword(APassword, qryUsers.FieldByName('PASSHASH').AsString);
  if not Result.Success then
    Exit;

  Result.Roles := qryUsers.FieldByName('ROLES').AsString.Split([',']);

  //ASubject.Expiration := IncSecond(Now(), 30);
  ASubject.Audience := 'delphiday';
  ASubject.UserID := qryUsers.FieldByName('ID').AsString;
end;

procedure TDMData.DataModuleCreate(Sender: TObject);
const
  DatabaseName = 'data.db';
begin
  inherited;
  FDConnection.DriverName := 'SQLite';
  FDConnection.Params.Add('Database=' + DatabaseName);
  FDConnection.Params.Add('SQLiteAdvanced=page_size=4096');
  FDConnection.Connected := True;
end;

end.
