{******************************************************************************}
{                                                                              }
{       WiRL: RESTful Library for Delphi                                       }
{                                                                              }
{       Copyright (c) 2015-2019 WiRL Team                                      }
{                                                                              }
{       https://github.com/delphi-blocks/WiRL                                  }
{                                                                              }
{******************************************************************************}
unit Server.Filters;

interface

uses
  WinApi.Windows, System.SysUtils, System.Classes,

  WiRL.Core.Registry,
  WiRL.http.Filters,
  WiRL.http.Request,
  WiRL.http.Response,
  WiRL.Core.Attributes,
  WiRL.Core.Exceptions,
  WiRL.Core.Auth.Context,
  WiRL.http.URL,
  WiRL.Core.Application,
  WiRL.http.Accept.MediaType;

type
  ILogger = interface
  ['{8F12BC74-8186-476D-8959-1A55A7097E02}']
    procedure Log(const AMessage: string);
  end;

  [PreMatching]
  TRequestLoggerFilter = class(TInterfacedObject, IWiRLContainerRequestFilter)
  private
    FLogger: ILogger;
  public
    procedure Filter(ARequestContext: TWiRLContainerRequestContext);
    constructor Create(ALogger: ILogger);
  end;

  TResponseLoggerFilter = class(TInterfacedObject, IWiRLContainerResponseFilter)
  private
    FLogger: ILogger;
  public
    procedure Filter(ARequestContext: TWiRLContainerResponseContext);
    constructor Create(ALogger: ILogger);
  end;

procedure RegisterLogger(ALogger: ILogger);
procedure ClearFilterRegistry;

implementation

{ TRequestLoggerFilter }

constructor TRequestLoggerFilter.Create(ALogger: ILogger);
begin
  FLogger := ALogger;
end;

procedure TRequestLoggerFilter.Filter(ARequestContext: TWiRLContainerRequestContext);
var
  LMessage: string;
begin
  LMessage := DateTimeToStr(Now) + ' - ' + ARequestContext.Request.Method + ' ' + ARequestContext.Request.PathInfo;
  if ARequestContext.Request.Query <> '' then
    LMessage := LMessage + '?' + ARequestContext.Request.Query;
  FLogger.Log('REQ - ' + LMessage);
end;

{ TResponseLoggerFilter }

constructor TResponseLoggerFilter.Create(ALogger: ILogger);
begin
  FLogger := ALogger;
end;

procedure TResponseLoggerFilter.Filter(
  ARequestContext: TWiRLContainerResponseContext);
var
  LMessage: string;
begin
  LMessage := DateTimeToStr(Now) + ' - ' + ARequestContext.Request.Method + ' ' + ARequestContext.Request.PathInfo;
  if ARequestContext.Request.Query <> '' then
    LMessage := LMessage + '?' + ARequestContext.Request.Query;
  FLogger.Log('RES - ' + LMessage);
end;

procedure RegisterLogger(ALogger: ILogger);
begin
  TWiRLFilterRegistry.Instance.RegisterFilter<TRequestLoggerFilter>(
    function (): TObject
    begin
      Result := TRequestLoggerFilter.Create(ALogger);
    end
  );
  TWiRLFilterRegistry.Instance.RegisterFilter<TResponseLoggerFilter>(
    function (): TObject
    begin
      Result := TResponseLoggerFilter.Create(ALogger);
    end
  );
end;

procedure ClearFilterRegistry;
begin
  TWiRLFilterRegistry.Instance.Clear;
end;

end.
