{******************************************************************************}
{                                                                              }
{       WiRL: RESTful Library for Delphi                                       }
{                                                                              }
{       Copyright (c) 2015-2019 WiRL Team                                      }
{                                                                              }
{       https://github.com/delphi-blocks/WiRL                                  }
{                                                                              }
{******************************************************************************}
unit Server.Forms.Main;

interface

uses
  System.Classes, System.SysUtils, Winapi.Windows, Vcl.Forms, Vcl.ActnList,
  System.Types, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.Controls, Vcl.ExtCtrls,
  System.Diagnostics, System.Actions, Winapi.ShellAPI,
  System.Generics.Collections, Winapi.Messages,
  Server.Core.Listener, Server.Filters;

type
  TMainForm = class(TForm, ILogger)
    TopPanel: TPanel;
    StartButton: TButton;
    StopButton: TButton;
    MainActionList: TActionList;
    StartServerAction: TAction;
    StopServerAction: TAction;
    PortNumberEdit: TEdit;
    Label1: TLabel;
    TestAction: TAction;
    Timer1: TTimer;
    Button1: TButton;
    memLog: TMemo;
    procedure StartServerActionExecute(Sender: TObject);
    procedure StartServerActionUpdate(Sender: TObject);
    procedure StopServerActionExecute(Sender: TObject);
    procedure StopServerActionUpdate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TestActionExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PortNumberEditChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FListener: TListener;
    FLogList: TThreadedQueue<string>;
    { ILogger }
    procedure Log(const AMessage: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  Server.Database.Builder;

constructor TMainForm.Create(AOwner: TComponent);
begin
  inherited;
  FListener := TListener.Create;
  FLogList := TThreadedQueue<string>.Create(100, INFINITE, 100);
  RegisterLogger(Self);
end;

destructor TMainForm.Destroy;
begin
  ClearFilterRegistry;
  FListener.Free;
  FLogList.Free;
  inherited;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  StartServerAction.Execute;
  TDatabaseBuilder.Initialize;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  StopServerAction.Execute;
end;

procedure TMainForm.Log(const AMessage: string);
begin
  FLogList.PushItem(AMessage);
end;

procedure TMainForm.PortNumberEditChange(Sender: TObject);
begin
  FListener.Port := StrToIntDef(PortNumberEdit.Text, 8080);
end;

procedure TMainForm.StartServerActionExecute(Sender: TObject);
begin
  FListener.Start;
end;

procedure TMainForm.StartServerActionUpdate(Sender: TObject);
begin
  StartServerAction.Enabled := not FListener.Active;
end;

procedure TMainForm.StopServerActionExecute(Sender: TObject);
begin
  FListener.Stop;
end;

procedure TMainForm.StopServerActionUpdate(Sender: TObject);
begin
  StopServerAction.Enabled := FListener.Active;
end;

procedure TMainForm.TestActionExecute(Sender: TObject);
const
  LTemplateUrl = 'http://localhost:%d/';
begin
  ShellExecute(Handle, 'open', PChar(Format(LTemplateUrl, [FListener.Port])), '', '', SW_NORMAL);
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
var
  LMessage: string;
begin
  while FLogList.PopItem(LMessage) = wrSignaled do
  begin
    memLog.Lines.Add(LMessage);
    // Si posiziona sull'ultima riga
    SendMessage(memLog.Handle, EM_LINESCROLL, 0, memLog.Lines.Count);
  end;
end;

initialization
  ReportMemoryLeaksOnShutdown := True;

end.
