unit Server.Resources.Auth;

interface

uses
  System.Classes, System.SysUtils, System.JSON, System.Generics.Collections, Data.DB,

  WiRL.Configuration.JWT,
  WiRL.Core.Engine,
  WiRL.Core.Application,
  WiRL.Core.Registry,
  WiRL.Core.Attributes,
  WiRL.Core.Validators,
  WiRL.http.Accept.MediaType,
  WiRL.http.URL,
  WiRL.Core.MessageBody.Default,
  WiRL.Core.Auth.Context,
  WiRL.http.Request,
  WiRL.http.Response,
  WiRL.Core.Auth.Resource,

  Server.Database.Data;

type
  [Path('auth')]
  TFormAuthResource = class(TWiRLAuthFormResource)
  private
    FDMData: TDMData;
  protected
    function Authenticate(const AUserName, APassword: string): TWiRLAuthResult; override;
  public
    constructor Create;
    destructor Destroy; override;

    [GET][Path('passhash')]//[RolesAllowed('admin')]
    [Produces(TMediaType.TEXT_PLAIN)]
    function GetUserPasswordHash([QueryParam('u')] const AUsername: string; [QueryParam('p')] const APassword: string): string;
  end;


implementation

{ TFormAuthResource }

function TFormAuthResource.Authenticate(const AUserName,
  APassword: string): TWiRLAuthResult;
begin
  Result := FDMData.Authenticate(AUserName, APassword, FAuthContext.Subject);
end;

constructor TFormAuthResource.Create;
begin
  FDMData := TDMData.Create(nil);
end;

destructor TFormAuthResource.Destroy;
begin
  FDMData.Free;
end;

function TFormAuthResource.GetUserPasswordHash(const AUsername,
  APassword: string): string;
begin
  Result := FDMData.GetHashedPassword(APassword);
end;

initialization
  TWiRLResourceRegistry.Instance.RegisterResource<TFormAuthResource>;

end.
