unit Server.Resources.Jobs;

interface

uses
  System.SysUtils, System.Classes, System.JSON, System.DateUtils, Data.DB,
  Variants, Windows, Datasnap.DBClient, ShellApi,

  WiRL.Core.JSON,
  WiRL.Core.Registry,
  WiRL.Core.Attributes,
  WiRL.http.Accept.MediaType,
  WiRL.http.URL,
  WiRL.http.Request,
  WiRL.Core.MessageBody.Default,
  WiRL.Data.MessageBody.Default,
  WiRL.Core.Exceptions,
  WiRL.Data.Resolver,

  Server.Database.Data;

type
  TJob = class
  private
    FId: Integer;
    FOperation: string;
  public
    property Id: Integer read FId write FId;
    property Operation: string read FOperation write FOperation;
  end;

  [Path('/jobs')]
  TJobsResource = class(TObject)
  private
    FDMData: TDMData;
  public
    [GET]
    [Produces(TMediaType.APPLICATION_JSON)]
    function GetJobs: TDataSet;

    [POST]
    [Path('/{operation}')]
    [RolesAllowed('users')]
    [Produces(TMediaType.APPLICATION_JSON)]
    function NewJobs([PathParam('operation')] const AOperation: string): TJob;

    constructor Create;
    destructor Destroy; override;
  end;


implementation

{ TJobsResource }

constructor TJobsResource.Create;
begin
  FDMData := TDMData.Create(nil);
end;

destructor TJobsResource.Destroy;
begin
  FDMData.Free;
  inherited;
end;

function TJobsResource.GetJobs: TDataSet;
begin
  Result := FDMData.GetJobs;
end;

function TJobsResource.NewJobs(const AOperation: string): TJob;
const
  WorkerName = 'DemoWorker.exe';
var
  LJobId: Integer;
begin
  LJobId := FDMData.NewJob(AOperation);
  ShellExecute(0, 'open', PChar(WorkerName), PChar(LJobId.ToString), '', 0);

  Result := TJob.Create;
  Result.id := LJobId;
  Result.Operation := AOperation;
end;

initialization

  TWiRLResourceRegistry.Instance.RegisterResource<TJobsResource>;

end.
